@@ -29,49 +29,6 @@
 (require 'rail-vars)
 (require 'rail-common)
 
-;; replace codename
-(defun rail-user-agent-replace-region (beg end pattern &optional pattern2)
-  "Replace codename in region."
-  (save-excursion
-    (save-restriction
-      (narrow-to-region beg end)
-      (goto-char (point-min))
-      (while (not (eobp))
-	(skip-chars-forward " \t\r\n")
-	(if (not (looking-at pattern))
-	    (skip-chars-forward "^ \t\r\n")
-	  (let* ((last (match-end 0))
-		 (kind (rail-assoc
-			(buffer-substring (match-beginning 1) (match-end 1))
-			rail-product-name-alist t))
-		 alist-a alist-b ccode)
- 	    (if (not kind)
- 		(goto-char last)
-	      (goto-char (match-beginning 2))
- 	      (setq ccode   (prin1-to-string kind)
- 		    alist-a (symbol-value
- 			     (intern (format "rail-additional-%s-codename-alist" ccode)))
- 		    alist-b (symbol-value
- 			     (intern (format "rail-%s-codename-alist" ccode))))
-	      (rail-replace-codename-primitive alist-a alist-b)
-	      (and pattern2
-		   (eq 'xmas kind)
-		   (looking-at pattern2)
-		   (let ((b2    (match-beginning 3))
-			 (e2    (match-end 3))
-			 (ccode (rail-assoc (rail-replace-into-iso8859-4
-					     (buffer-substring b2 e2))
-					    (append rail-additional-xmas-codename-alist
-						    rail-xmas-codename-alist)
-					    rail-convert-direction)))
-		     (if (not ccode)
-			 (goto-char last)
-		       (goto-char b2)
-		       (delete-region b2 e2)
-		       (insert ccode)
-		       (forward-char 1))))))))
-      (buffer-substring (point-min) (point-max)))))
-
 ;; search header
 (defun rail-user-agent-search-header (head)
   "Search header string and return region."
@@ -90,30 +47,75 @@
 		   (cons pt (point)))))))))
 
 ;; replace MIME-Version: header dinamically
-(defun rail-user-agent-replace-mime-version-region (beg end)
+(defun rail-user-agent-replace-mime-version-region (region)
   "Replace codename in MIME-Version: region."
-  (rail-user-agent-replace-region beg end rail-mime-version-header-format))
+  (and region
+       (car region)
+       (cdr region)
+       (save-restriction
+	 (narrow-to-region (car region) (cdr region))
+	 (goto-char (point-min))
+	 (and (looking-at rail-mime-version-header-format)
+	      (rail-replace-codename-primitive
+	       rail-mime-version-header-format
+	       rail-additional-semi-codename-alist rail-semi-codename-alist))
+	 (buffer-substring (point-min) (point-max)))))
 
 ;; replace User-Agent: header dinamically
-(defun rail-user-agent-replace-user-agent-region (beg end)
+(defun rail-user-agent-replace-user-agent-region (region)
   "Replace codename in User-Agent: region."
-  (rail-user-agent-replace-region
-   beg end rail-user-agent-header-format rail-user-agent-header-xmas-format))
+  (and region
+       (car region)
+       (cdr region)
+       (save-restriction
+	 (narrow-to-region (car region) (cdr region))
+	 (goto-char (point-min))
+	 (while (not (eobp))
+	   (skip-chars-forward " \t\r\n")
+	   (if (not (looking-at "\\([^ \t\r\n/]+\\)/"))
+	       (skip-chars-forward "^ \t\r\n")
+	     (let ((kind (rail-assoc
+			  (upcase (buffer-substring (match-beginning 1) (match-end 1)))
+			  rail-product-name-alist t)))
+	       (if (not kind)
+		   (skip-chars-forward "^ \t\r\n")
+		 (goto-char (match-end 0))
+		 (or (and (eq 'xmas kind)
+			  (looking-at rail-user-agent-header-xmas-format)
+			  (rail-replace-codename-primitive
+			   rail-user-agent-header-xmas-format
+			   rail-additional-xmas-codename-alist rail-xmas-codename-alist))
+		     (let* ((skind (prin1-to-string kind))
+			    (alist-a
+			     (intern (format "rail-additional-%s-codename-alist" skind)))
+			    (alist-b
+			     (intern (format "rail-%s-codename-alist" skind))))
+		       (and (looking-at rail-user-agent-header-format)
+			    (boundp alist-a)
+			    (boundp alist-b)
+			    (rail-replace-codename-primitive
+			     rail-user-agent-header-format
+			     (symbol-value alist-a) (symbol-value alist-b)))))
+		 (and (eq 'meadow kind)
+		      (rail-replace-codename-meadow))))))
+	 (buffer-substring (point-min) (point-max)))))
 
 ;; replace header with rail-user-agent-replace-{mime-version|user-agent}-region
 (defun rail-user-agent-translate-body (&optional buf)
   "Translate message body. It executes after mime-edit-translate-body."
   (and rail-user-agent-convert-dynamically
-       (let (region)
-	 (save-excursion
-	   (and buf (set-buffer buf))
-	   (setq region (rail-user-agent-search-header "MIME-Version"))
-	   (rail-user-agent-replace-mime-version-region (car region) (cdr region))
-	   (mapcar
-	    '(lambda (item)
-	       (and (setq region (rail-user-agent-search-header item))
-		    (rail-user-agent-replace-user-agent-region (car region) (cdr region))))
-	    rail-user-agent-header-item-list)))))
+       (save-excursion
+	 (and buf (set-buffer buf))
+	 (mapcar
+	  '(lambda (item)
+	     (rail-user-agent-replace-mime-version-region
+	      (rail-user-agent-search-header item)))
+	  rail-mime-version-header-item-list)
+	 (mapcar
+	  '(lambda (item)
+	     (rail-user-agent-replace-user-agent-region
+	      (rail-user-agent-search-header item)))
+	  rail-user-agent-header-item-list))))
 
 ;; replace statically
 (defun rail-user-agent-replace-string ()
@@ -128,12 +130,14 @@
 		 (erase-buffer)
 		 (insert mime-edit-mime-version-value)
 		 (setq mime-edit-mime-version-value
-		       (rail-user-agent-replace-mime-version-region (point-min) (point-max)))
+		       (rail-user-agent-replace-mime-version-region
+			(cons (point-min) (point-max))))
 		 ;; mime-edit-user-agent-value
 		 (erase-buffer)
 		 (insert mime-edit-user-agent-value)
 		 (setq mime-edit-user-agent-value
-		       (rail-user-agent-replace-user-agent-region (point-min) (point-max)))
+		       (rail-user-agent-replace-user-agent-region
+			(cons (point-min) (point-max))))
 		 ;; end
 		 (kill-buffer buf))))))
   ;; mime-library-product
